; FFT spectrum analyzer 0 --> 5000 Hz
; 500 Hz/div. on graticule
; auto-tracking peak detector
; resolution = sample rate / number of samples (5.38 Hz)


InitSprite()
;InitSprite3D()
InitSound()
InitMovie()
InitKeyboard()
InitMouse()

RightOffset = 300

#SAMPLE_RATE   = 44100
#N_SAMPLES     = 8192 ;NUMBER OF SAMPLES

#WINDOW_WIDTH  = 1920 ;+ 300 ; full scale (5000) / resolution
#WINDOW_HEIGHT = 1080
#TRACE_COLOR   = $20DA42
#BACK_COLOR    = $004720
#GRID_COLOR    = 0
#Canvas        = 0
#Graticule     = 0 ; image



Structure MYWAVEFORMATEX
  wFormatTag.u ;UNSIGNED FOR COMPATIBILITY WITH MICROSOFT FLAG VALUES
  nChannels.w
  nSamplesPerSec.l
  nAvgBytesPerSec.l
  nBlockAlign.w
  wBitsPerSample.w
  cbSize.w
EndStructure

Global FFTWnd, smoothing, initSmoothing = #True

Structure CONFIG
  hWindow.i           ; Window handle
  size.i              ; Wave buffer size
  buffer.i            ; Wave buffer pointer
  wave.i              ; Address of waveform-audio input device
  format.MYWAVEFORMATEX ; Capturing WaveFormatEx
  lBuf.i                ; Capturing Buffer size
  nBuf.i                ; Capturing Buffer number
  nDev.i                ; Capturing Device identifier
  nBit.i                ; Capturing Resolution (8/16)
  nHertz.i              ; Capturing Frequency  (Hertz)
  nChannel.i            ; Capturing Channels number (Mono/Stereo)
  
EndStructure


Structure Punkt
  X.w
  Y.w
  R.l
  G.l
  B.l
  Direct.l
  Speed.f
  Sprite.l
  Dead.l
  DeadDelay.l
EndStructure
NewList Punkt.Punkt()


Global Config.CONFIG
Global Dim inHdr.WAVEHDR(16)
Global NumDevsMax.l
Global Dev.l
Config\format\wFormatTag = #WAVE_FORMAT_PCM

Procedure CAPTURE_START()
  
  waveInStop_(Config\wave)
  
  NumDevsMax = waveInGetNumDevs_()
  
  Config\format\nChannels=1
  Config\format\wBitsPerSample = 16
  Config\format\nSamplesPerSec = #SAMPLE_RATE
  Config\nDev = Dev;0 ; (default MS Sound Mapper device)
  Config\lBuf = #N_SAMPLES
  Config\nBuf = 8
  Config\nBit = 1
  
  ;Debug Config\nDev
  
  Config\format\nBlockAlign = (Config\format\nChannels * Config\format\wBitsPerSample)/8
  Config\format\nAvgBytesPerSec = Config\format\nSamplesPerSec * Config\format\nBlockAlign
  
  If #MMSYSERR_NOERROR = waveInOpen_(@Config\wave, #WAVE_MAPPER+Config\nDev, @Config\format,Config\hWindow, #Null, #CALLBACK_WINDOW | #WAVE_FORMAT_DIRECT)
    
    
    For i=0 To Config\nBuf-1
      inHdr(i)\lpData=AllocateMemory(Config\lBuf)
      inHdr(i)\dwBufferLength=Config\lBuf
      waveInPrepareHeader_(Config\wave, inHdr(i), SizeOf(WAVEHDR))
      waveInAddBuffer_(Config\wave, inHdr(i), SizeOf(WAVEHDR))
    Next
    
    
    waveInStart_(Config\wave)
    
  EndIf
  
  changedDev = 0
EndProcedure


EnableExplicit
Procedure COMPLEX_FFT()
  ; http://paulbourke.net/miscellaneous/dft/
  ; Appendix B. FFT (Fast Fourier Transform)
  ; This computes an in-place complex-To-complex FFT
  ; Real And Imag are the real And imaginary arrays of 2^m points.
  
  Static Dim Real.d(#N_SAMPLES)
  Static Dim Imag.d(#N_SAMPLES)
  Static Dim SmSum(#WINDOW_WIDTH-1)
  Static Dim Sm_Buff(#WINDOW_WIDTH-1,31) ; max smoothing = 32
  Static SmIDX = 0
  Static NM1 = #N_SAMPLES - 1
  Static ND2 = #N_SAMPLES >> 1
  Static Hscale.f = 0.0 + (#SAMPLE_RATE / #N_SAMPLES)
  Global Ergebnis.f
  
  
  Protected mask = smoothing - 1
  ;Protected M = Int(Log(#N_SAMPLES) / Log(2))
  Protected.i M = Int(Log(#N_SAMPLES) / 0.69314718055994529)
  Protected.i i, IP, j, k, n, LE2, LE, Imax, y, Ly, peak_Y, peak_X
  Protected.d C1, C2, TR, TI, UR, UI, ZZ
  
  If initSmoothing : initSmoothing = #False
    i = 0 ; init smoothing buffers
    While i < #WINDOW_WIDTH : k = 0
      While k < smoothing : Sm_Buff(i,k) = #WINDOW_HEIGHT : k + 1 : Wend
      SmSum(i) = #WINDOW_HEIGHT * smoothing
      i + 1
    Wend
  EndIf
  
  ; -------- Clear and Fill array values for analyzing --------
  i = 0 : j = ND2 : k = 0
  While k < Config\size
    Real(i) = PeekW(Config\buffer + k) ; 0 to SAMPLES/2 - 1
    Imag(i) = 0                        ; 0 to SAMPLES/2 - 1
    Real(j) = 0                        ; SAMPLES/2 to SAMPLES
    Imag(j) = 0                        ; SAMPLES/2 to SAMPLES
    i + 1 : j + 1 : k + 2
  Wend
  
  j = 0
  Imax = #N_SAMPLES - 2
  
  For i = 0 To Imax  ; Gold-Rader bit reversal algorithm
    If i < j
      Swap Real(i),Real(j)
      ;Swap Imag(i), Imag(j) ; both are zero
    EndIf
    k = ND2
    While k <= j
      j = j - k
      k = k >> 1
    Wend
    j = j + k
  Next
  
  
  
  
  ; Compute the FFT
  C1 = -1.0
  C2 = 0.0
  LE = 1
  
  For n = 1 To M
    LE2 = LE
    LE << 1
    UR = 1.0
    UI = 0.0
    
    k = 0
    While k < LE2 : i = k
      While i < #N_SAMPLES ; Loop for each butterfly calculation
        IP = i + LE2
        TR = Real(IP) * UR - Imag(IP) * UI
        TI = Imag(IP) * UR + Real(IP) * UI
        Real(IP) = Real(i) - TR
        Imag(IP) = Imag(i) - TI
        Real(i) + TR
        Imag(i) + TI
        i + LE
      Wend
      
      ZZ = UR * c1 - UI * C2
      UI = UR * c2 + UI * C1
      UR = ZZ
      k + 1
    Wend
    
    C2 = Sqr((1.0 - C1) / 2.0)
    C1 = Sqr((1.0 + C1) / 2.0)
  Next n
  
  For i = 0 To NM1 ; Scaling For forward transform
    Real(i) = Abs(Real(i) / #N_SAMPLES)
    ;Imag(i) = Abs(Imag(i) / #N_SAMPLES) ; we don't use this
  Next
  
  
  
  Ly = #WINDOW_HEIGHT
  
  i = 0 : peak_Y = #WINDOW_HEIGHT
  While i < #WINDOW_WIDTH
    y = #WINDOW_HEIGHT - Log(Real(i)+1) * 64 ; 64 is arbitrary
    If smoothing
      SmSum(i) - Sm_Buff(i,SmIDX)
      Sm_Buff(i,SmIDX) = y
      SmSum(i) + y
      y = SmSum(i) / smoothing ; this is the moving average
      
    EndIf
    
    ;          LineXY(i-1,Ly,i,y,#TRACE_COLOR) : Ly = y
    
    If y < peak_Y : peak_Y = y : peak_X = i : EndIf
    i + 1
  Wend
  
  ;       Circle(peak_X, peak_Y, 3, #Red)
  Ergebnis = peak_X * Hscale;+" Hz"
  
  If smoothing : SmIDX = (SmIDX + 1) & mask : EndIf
  ;   StopDrawing()
  
  ProcedureReturn #True
EndProcedure
DisableExplicit


Procedure CAPTURE_CALLBACK(hWnd.i, Msg.i, wParam.i, lParam.i)
  If Msg = #MM_WIM_DATA
    *hWave.WAVEHDR = lParam
    Config\buffer = *hWave\lpData
    Config\size = *hWave\dwBytesRecorded
    waveInAddBuffer_(wParam, *hWave, SizeOf(WAVEHDR))
    
    COMPLEX_FFT()
  EndIf
  
  ProcedureReturn #PB_ProcessPureBasicEvents
EndProcedure



Define flags = #PB_Window_ScreenCentered  | #PB_Window_Maximize | #PB_Window_BorderLess  
Define title.s = "FFT Spectrum | horizontal = 500Hz per division"
Define  xx.f, yy.f, inc.f


RightOffset = 0;300
FrameRate = 60

If OpenWindow(0, 0, 0, #WINDOW_WIDTH, #WINDOW_HEIGHT, title, flags)
  If OpenWindowedScreen(WindowID(0), 0, 0,#WINDOW_WIDTH, #WINDOW_HEIGHT,0,RightOffset,0)
    
  ;- StartVar
    
    MaxPitchDiffUmschalt = 200
    
    
    Alpha2 = 255
    RR = 255
    GG = 0
    BB = 255
    Delay = 1   
    ElipseZoomMax = 240
    OffsetX = WindowWidth(0) - RightOffset
    SAlpha = 200
    SAlphaMax = 5
    FadeSpriteDelayMax = 10
    
    ReadFile(0,"StartSettings.txt")
    If IsFile(0) = 0
      Source = 7 ;SpriteBlending Mode Source
      Destination = 3
      StartSource = 7
      StartDestination = 3
    
      SourceE = 7;SpriteBlendingMode Ellipse
      DestinationE = 8
      StartSourceE = 7
      StartDestinationE = 8
    EndIf
    
    ;R = Random(255)
    ;G = Random(200)
    ;B = 0;Random(255)
    
    DelMax = 100
    Del = DelMax - 10
    
    
    ClearscreenMax = 250
    ErgebnisMin = 50
    Zoomer = ErgebnisMin
    
    ModeMax = 4

    
    
    ;If 1 = 0
    ReadFile(0,"StartSettings.txt")
      If IsFile(0) <> 0  
        StartDev = ReadLong(0)
        Dev = ReadLong(0)
        Mode = ReadLong(0)
        Source = ReadLong(0)
        Destination = ReadLong(0)
        SourceE = ReadLong(0)
        DestinationE = ReadLong(0)
        StartMode = ReadLong(0)
        StartMode = Mode
        StartSource = Source
        StartDestination = Destination
        StartSourceE = SourceE
        StartDestinationE = DestinationE
      EndIf   
      CloseFile(0)
      
      If Mode = 0
          R = Random(255)
          G = Random(R)-50
          If G < 0
            G = 0
          EndIf 
          B = 0;Random(255)
        ElseIf Mode = 1
          R = Random(255)
          G = Random(255)
          B = 0
        ElseIf Mode = 2
          R = Random(255)
          G = Random(255)
          B = Random(255)
        ElseIf Mode = 3
          R = 0
          G = 0
          B = Random(155)
        ElseIf Mode = 4
          a = Random(3)
          If a = 0
            R = Random(255)
            G = 0
            B = 0
          ElseIf a = 1
            R = 0
            G = 0
            B = Random(255)
          ElseIf a = 2
            R = Random(255)
            G = Random(255)
            B = Random(255)
          ElseIf a = 3
            R = 255
            G = 255
            B = 255
          EndIf 
        EndIf
        
        
    ;EndIf   
      
    CreateSprite(10002,ErgebnisMin,ErgebnisMin,#PB_Sprite_AlphaBlending)
    StartDrawing(SpriteOutput(10002))
    Box(0,0,ErgebnisMin,ErgebnisMin,RGB(0,0,0))
    Ellipse(ErgebnisMin/2,ErgebnisMin/2,ErgebnisMin/2,ErgebnisMin/2,RGB(R,G,B))
    StopDrawing()
    
    
    
    CreateSprite(10005,#WINDOW_WIDTH,#WINDOW_HEIGHT,#PB_Sprite_AlphaBlending)
    
    
    
    Config\hWindow = WindowID(0)
    
    
    
    SetWindowCallback(@CAPTURE_CALLBACK())
    CAPTURE_START()
    
    
    ;- Schleife
    
    Repeat ; Screen
      
      
      ;- AddPunkt
      ;PunktFarbDelay + 1 
      PunktDelay + 1  
      If Ergebnis > ErgebnisMin And PunktDelay >= 10
        If PunktSchleifenCount = 3
          PR = R
          PG = G
          PB = B
        EndIf 
        
        For D = 1 To 23 Step 2
          AddElement(Punkt())
          Punkt()\X = #WINDOW_WIDTH/2-3
          Punkt()\Y = #WINDOW_HEIGHT/2-3
          Punkt()\R = PR
          Punkt()\G = PG
          Punkt()\B = PB
          Punkt()\Direct = D
          Punkt()\Speed = Ergebnis/64
          Punkt()\DeadDelay = 1000
          CreateSprite(Punkt,6,6)
          StartDrawing(SpriteOutput(Punkt))
          ;DrawingMode(#PB_2DDrawing_Outlined)
          Circle(3,3,2,RGB(Punkt()\R,Punkt()\G,Punkt()\B))
          StopDrawing()
          Punkt()\Sprite = Punkt
          Punkt + 1
        Next 
        
        For D = 24 To 72; Step 2
          AddElement(Punkt())
          Punkt()\X = #WINDOW_WIDTH/2-3
          Punkt()\Y = #WINDOW_HEIGHT/2-3
          Punkt()\R = PR
          Punkt()\G = PG
          Punkt()\B = PB
          Punkt()\Direct = D
          Punkt()\Speed = Ergebnis/64
          Punkt()\DeadDelay = 1000
          CreateSprite(Punkt,6,6)
          StartDrawing(SpriteOutput(Punkt))
          DrawingMode(#PB_2DDrawing_Outlined)
          Circle(3,3,2,RGB(Punkt()\R,Punkt()\G,Punkt()\B))
          StopDrawing()
          Punkt()\Sprite = Punkt
          Punkt + 1
        Next
        
        
        If Punkt > 1000
          Punkt = 0
        EndIf
        
        PunktSchleifenCount + 1
        PunktDelay = 0
        If PunktSchleifenCount = 16
          PunktSchleifenCount = 0
        EndIf 
      EndIf
      
      
      
      Repeat ; Window
        
        OffsetX = WindowWidth(0) - RightOffset
        
        
        
        
        Event = WaitWindowEvent(1)
        
        
        
        If Event = #PB_Event_CloseWindow
          End
        EndIf
      Until Event = 0
      
      
      ScreenClearer + 1
      
      
      ;SpriteBlendingMode(7,8)
      If FadeSpriteDelay = 0
        ;DisplayTransparentSprite(10004,0,0,200) ;Gradient-Fader_Sprite
        FadeSpriteDelay = FadeSpriteDelayMax
      EndIf 
      If FadeSpriteDelay > 0
        FadeSpriteDelay - 1
      EndIf
      
      
      FlipBuffers()
      
      
      ;- Farbmodi
      
      If Dif > MaxPitchDiffUmschalt/4
        ;ScreenClearer = ClearscreenMax
        If Mode = 0
          R = Random(255)
          G = Random(R)-50
          If G < 0
            G = 0
          EndIf 
          B = 0;Random(255)
        ElseIf Mode = 1
          R = Random(255)
          G = Random(255)
          B = 0
        ElseIf Mode = 2
          R = Random(255)
          G = Random(255)
          B = Random(255)
        ElseIf Mode = 3
          R = 0
          G = 0
          B = Random(155)
        ElseIf Mode = 4
          a = Random(3)
          If a = 0
            R = Random(255)
            G = 0
            B = 0
          ElseIf a = 1
            R = 0
            G = 0
            B = Random(255)
          ElseIf a = 2
            R = Random(255)
            G = Random(255)
            B = Random(255)
          ElseIf a = 3
            R = 255
            G = 255
            B = 255
          EndIf 
        EndIf 
      ElseIf Dif < (MaxPitchDiffUmschalt*-1)/4
        ;ScreenClearer = ClearscreenMax
        If Mode = 0
          R = Random(255)
          G = Random(R)-50
          If G < 0
            G = 0
          EndIf 
          B = 0;Random(255)
        ElseIf Mode = 1
          R = Random(255)
          G = Random(255)
          B = 0
        ElseIf Mode = 2
          R = Random(255)
          G = Random(255)
          B = Random(255)
        ElseIf Mode = 3
          R = 0
          G = 0
          B = Random(255)
        ElseIf Mode = 4
          a = Random(3)
          If a = 0
            R = Random(255)
            G = 0
            B = 0
          ElseIf a = 1
            R = Random(255)
            G = 0
            B = Random(255)
          ElseIf a = 2
            R = Random(255)
            G = 0
            B = Random(255)
          ElseIf a = 3
            R = 255
            G = 255
            B = 255
          EndIf 
        EndIf 
      EndIf
      
      
      ;       If Dif > MaxPitchDiffUmschalt
      ;         ScreenClearer = ClearscreenMax
      ;       ElseIf Dif < (MaxPitchDiffUmschalt*-1) ;Erlaubt schnelle Clearscreen-Wechsel bei drastischen Pegel-Schwankungen (Epilepsiewarnung angebracht/wirkt hecktisch)
      ;         ScreenClearer = ClearscreenMax
      ;       EndIf
      
      If SAlpha < 255
        SAlpha + 10
      EndIf 
      
      ;-ScreenClearer       
      If ScreenClearer >= ClearscreenMax
        
        If Ergebnis >= 1500
          RR = Random(55)+200
          GG = Random(55)+100
          BB = Random(55)+100
          If RR >= GG And RR >= BB
            ClearscreenMax = RR
            If GG > BB
              RR = Random(55)+100
              GG = Random(60)
            ElseIf BB > GG
              RR = Random(55)+100
              BB = Random(60)
            EndIf
            
          ElseIf GG >= RR And GG >= BB
            ClearscreenMax = GG
          Else
            ClearscreenMax = BB
          EndIf
          
          ScreenClearer = 0
          SAlpha = SAlphaMax
          ClearScreen(RGB(0,0,0));RGBA(RR,GG,BB,SAlpha)
          FadeSpriteDelay = 0
          
        ElseIf Ergebnis < 1500 And Ergebnis >= 1000
          RR = Random(70)+70
          GG = Random(70)+70
          BB = Random(70)+70
          If RR >= GG And RR >= BB
            ClearscreenMax = RR
          ElseIf GG >= RR And GG >= BB
            ClearscreenMax = GG
          Else
            ClearscreenMax = BB
          EndIf
          
          ScreenClearer = 0
          SAlpha = SAlphaMax
          ClearScreen(RGB(0,0,0));RGBA(RR,GG,BB,SAlpha)
          FadeSpriteDelay = 0
          
        ElseIf Ergebnis < 1000 And Ergebnis >= 700
          RR = Random(55)+45
          GG = Random(55)+45
          BB = Random(55)+45
          If RR >= GG And RR >= BB
            ClearscreenMax = RR
          ElseIf GG >= RR And GG >= BB
            ClearscreenMax = GG
          Else
            ClearscreenMax = BB
          EndIf
          
          ScreenClearer = 0
          SAlpha = SAlphaMax
          ClearScreen(RGB(0,0,0));RGBA(RR,GG,BB,SAlpha)
          FadeSpriteDelay = 0
          
        ElseIf Ergebnis < 700 And Ergebnis > ErgebnisMin
          RR = Random(25)+15
          GG = Random(25)+15
          BB = Random(25)+15
          If RR >= GG And RR >= BB
            ClearscreenMax = RR
          ElseIf GG >= RR And GG >= BB
            ClearscreenMax = GG
          Else
            ClearscreenMax = BB
          EndIf
          
          ScreenClearer = 0
          SAlpha = SAlphaMax
          ClearScreen(RGB(0,0,0));RGBA(RR,GG,BB,SAlpha)
          FadeSpriteDelay = 0
          
        EndIf
        
        
        
      EndIf
      
      
      
      
      Zoom = Ergebnis
      If Zoom > 700
        Zoom = 700
      EndIf
      If Zoom < ErgebnisMin
        Zoom = ErgebnisMin
      EndIf
      
      
      
      StartDrawing(SpriteOutput(10002))
      Box(0,0,ErgebnisMin,ErgebnisMin,RGB(0,0,0))
      Ellipse(ErgebnisMin/2,ErgebnisMin/2,ErgebnisMin/2,ErgebnisMin/2,RGB(R,G,B))
      StopDrawing()
      
      
      
      StartDrawing(ScreenOutput())
      
      ;- DebugText 
      ;DrawText(0,0,"changedDev: "+Str(changedDev)+" Device: "+Str(Dev),RGB(255,255,0))
      
      If Ergebnis <= ErgebnisMin 
        Ergebnis = ErgebnisMin
        CAlpha = 25
      EndIf
      
      ;- Circles 
      If Ergebnis > ErgebnisMin And Ergebnis < 200
        CAlpha = 150
      ElseIf Ergebnis >= 200 And Ergebnis < 500
        CAlpha = 75
      Else
        CAlpha = 10
      EndIf
      DrawingMode(#PB_2DDrawing_AlphaBlend| #PB_2DDrawing_Outlined)
      
      If Ergebnis < #WINDOW_HEIGHT/2
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis*2,RGBA(PR,PG,PB,CAlpha))
      EndIf 
      If Ergebnis > 32
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis-8,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis-16,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis-32,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis/2,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis/4,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis/6,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis/8,RGBA(PR,PG,PB,CAlpha))
        Circle(#WINDOW_WIDTH/2,#WINDOW_HEIGHT/2,Ergebnis/10,RGBA(PR,PG,PB,CAlpha))
      EndIf     
      
      
      ;CountPunkte = ListSize(Punkt())
      Dif = Ergebnis - LastErgebnis
      
      StopDrawing()
      
      ReleaseMouse(0)
      
      
      
      ;- Punkt 
      SpriteBlendingMode(Source,Destination);(7,3)
      SpriteQuality(1)
      
      ResetList(Punkt())
      While NextElement(Punkt())
        
        If Punkt()\Direct = 0
          ;Punkt()\Y - Punkt()\Speed/Cos(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 1
          Punkt()\Y - Punkt()\Speed/Cos(#PI+Ergebnis/10)
          Punkt()\X + Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 2
          ;Punkt()\X + Punkt()\Speed/Cos(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 3
          Punkt()\Y + Punkt()\Speed/Cos(#PI+Ergebnis/10)
          Punkt()\X + Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 4
          ;Punkt()\Y + Punkt()\Speed/Cos(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 5
          Punkt()\Y + Punkt()\Speed/Cos(#PI+Ergebnis/10)
          Punkt()\X - Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 6
          ;Punkt()\X - Punkt()\Speed/Cos(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 7
          Punkt()\Y - Punkt()\Speed/Cos(#PI+Ergebnis/10)
          Punkt()\X - Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 8
          ;Punkt()\Y - Punkt()\Speed/Sin(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 9
          Punkt()\Y - Punkt()\Speed/Sin(#PI+Ergebnis/10)
          Punkt()\X + Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 10
          ;Punkt()\X + Punkt()\Speed/Sin(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 11
          Punkt()\Y + Punkt()\Speed/Sin(#PI+Ergebnis/10)
          Punkt()\X + Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 12
          ;Punkt()\Y + Punkt()\Speed/Sin(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 13
          Punkt()\Y + Punkt()\Speed/Sin(#PI+Ergebnis/10)
          Punkt()\X - Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 14
          ;Punkt()\X - Punkt()\Speed/Sin(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 15
          Punkt()\Y - Punkt()\Speed/Sin(#PI+Ergebnis/10)
          Punkt()\X - Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 16
          Punkt()\Y - Punkt()\Speed/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 17
          Punkt()\Y - Punkt()\Speed/Tan(#PI+Ergebnis/10)
          Punkt()\X + Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 18
          Punkt()\X + Punkt()\Speed/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 19
          Punkt()\Y + Punkt()\Speed/Tan(#PI+Ergebnis/10)
          Punkt()\X + Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 20
          Punkt()\Y + Punkt()\Speed/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 21
          Punkt()\Y + Punkt()\Speed/Tan(#PI+Ergebnis/10)
          Punkt()\X - Punkt()\Speed;/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 22
          Punkt()\X - Punkt()\Speed/Tan(#PI+Ergebnis/10)
        ElseIf Punkt()\Direct = 23
          Punkt()\Y - Punkt()\Speed/Tan(#PI+Ergebnis/10)
          Punkt()\X - Punkt()\Speed;/Tan(#PI+Ergebnis/10)
          
          
          
        ElseIf Punkt()\Direct = 24
          Punkt()\Y + Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 25
          Punkt()\Y + Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 26
          Punkt()\Y - Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 27
          Punkt()\Y - Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 28
          Punkt()\Y + Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 29
          Punkt()\Y + Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 30
          Punkt()\Y - Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 31
          Punkt()\Y - Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Cos(#PI+Ergebnis)
          
          
          
        ElseIf Punkt()\Direct = 32
          Punkt()\Y + Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 33
          Punkt()\Y + Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 34
          Punkt()\Y - Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 35
          Punkt()\Y - Punkt()\Speed+Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Tan(#PI+Ergebnis)
          
          
          
        ElseIf Punkt()\Direct = 37
          Punkt()\Y + Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 38
          Punkt()\Y + Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 39
          Punkt()\Y - Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 40
          Punkt()\Y - Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Tan(#PI+Ergebnis)
          
          
          
          
          
        ElseIf Punkt()\Direct = 41
          Punkt()\Y + Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 42
          Punkt()\Y + Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 43
          Punkt()\Y - Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 44
          Punkt()\Y - Punkt()\Speed+Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Sin(#PI+Ergebnis)
          
          
          
          
          
        ElseIf Punkt()\Direct = 45
          Punkt()\Y + Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 46
          Punkt()\Y + Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 47
          Punkt()\Y - Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed+Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 48
          Punkt()\Y - Punkt()\Speed+Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed+Sin(#PI+Ergebnis)
          
          
          
          ;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          
        ElseIf Punkt()\Direct = 49
          Punkt()\Y + Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 50
          Punkt()\Y + Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 51
          Punkt()\Y - Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 52
          Punkt()\Y - Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 53
          Punkt()\Y + Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 54
          Punkt()\Y + Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 55
          Punkt()\Y - Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Cos(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 56
          Punkt()\Y - Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Cos(#PI+Ergebnis)
          
          
          
        ElseIf Punkt()\Direct = 57
          Punkt()\Y + Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 58
          Punkt()\Y + Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 59
          Punkt()\Y - Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 60
          Punkt()\Y - Punkt()\Speed-Sin(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Tan(#PI+Ergebnis)
          
          
          
        ElseIf Punkt()\Direct = 61
          Punkt()\Y + Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 62
          Punkt()\Y + Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 63
          Punkt()\Y - Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Tan(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 64
          Punkt()\Y - Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Tan(#PI+Ergebnis)
          
          
          
          
          
        ElseIf Punkt()\Direct = 65
          Punkt()\Y + Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 66
          Punkt()\Y + Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 67
          Punkt()\Y - Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 68
          Punkt()\Y - Punkt()\Speed-Cos(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Sin(#PI+Ergebnis)
          
          
          
          
          
        ElseIf Punkt()\Direct = 69
          Punkt()\Y + Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 70
          Punkt()\Y + Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 71
          Punkt()\Y - Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X - Punkt()\Speed-Sin(#PI+Ergebnis)
        ElseIf Punkt()\Direct = 72
          Punkt()\Y - Punkt()\Speed-Tan(#PI+Ergebnis)
          Punkt()\X + Punkt()\Speed-Sin(#PI+Ergebnis)
        EndIf
        
        
        
        
        
        Punkt()\DeadDelay - 1
        
        If Punkt()\DeadDelay <= 0
          Punkt()\Dead + 1
        EndIf
        
        If Punkt()\X < 0- SpriteWidth(Punkt()\Sprite)
          Punkt()\Dead + 1
        EndIf
        If Punkt()\X > #WINDOW_WIDTH
          Punkt()\Dead + 1
        EndIf
        If Punkt()\Y < 0 - SpriteHeight(Punkt()\Sprite)
          Punkt()\Dead + 1
        EndIf
        If Punkt()\Y > #WINDOW_HEIGHT
          Punkt()\Dead + 1
        EndIf
        
        
        
        If Punkt()\Dead <= 0
          DisplayTransparentSprite(Punkt()\Sprite,Punkt()\X,Punkt()\Y,0)
        EndIf
        
        
        
        
        If Punkt()\Dead > 0
          DeleteElement(Punkt())
        EndIf
        
      Wend
      
      LastErgebnis = Ergebnis
      ;- Elipse
      
      
      
      
      If Zoom > AltZoom
        Zoomer = Zoom
        Rotater + 5
      ElseIf Zoom < AltZoom
        Zoomer - 10
        If Rotater > 5
          Rotater - 5
        EndIf
        
      EndIf
      
      If Zoomer > ElipseZoomMax
        Zoomer = ElipseZoomMax
      EndIf
      If Zoomer < ErgebnisMin
        Zoomer = ErgebnisMin
      EndIf
      
      
      
      AltZoom = Zoom
      
      
      
      
      
      ;- Start3d 
      ;Start3D()
      
      SpriteQuality(1)
      ZoomSprite(10002,Zoomer,Zoomer/2)
      If Rotater > 360
        Rotater = 360
      EndIf
      If Rotater < 0
        Rotater = 0
      EndIf
      
      RotateSprite(10002,Rotater,#PB_Relative)
      SpriteBlendingMode(SourceE,DestinationE);(7,8)
      DisplayTransparentSprite(10002,(#WINDOW_WIDTH/2)-Zoomer/2,(#WINDOW_HEIGHT/2)-Zoomer/4,200) ;Elipse
      
      
      
      
      ;- Keyboard
      
      ExamineKeyboard()
      If KeyboardReleased(#PB_Key_Up) ;Wechsle durch die verschiedenen Farb-Modi (Beim hinzufgen zustzlicher Modi, bitte den ModeMax Wert anpassen - von Haus aus auf 4 - also 5 verschiedene Modi)
        Mode + 1
      EndIf 
      If KeyboardReleased(#PB_Key_Down)  ;Wechsle durch die verschiedenen Farb-Modi (Beim hinzufgen zustzlicher Modi, bitte den ModeMax Wert anpassen - von Haus aus auf 4 - also 5 verschiedene Modi)
        Mode - 1
      EndIf 
      If Mode > ModeMax
        Mode = 0
      EndIf 
      If Mode < 0
        Mode = ModeMax
      EndIf 
      
      If KeyboardReleased(#PB_Key_W) ;Erhht Source-Wert von SpriteBlendingMode der 'Punkte'
        Source + 1
      EndIf 
      If KeyboardReleased(#PB_Key_S) ;Verringert Source-Wert von SpriteBlendingMode der 'Punkte'
        Source - 1
      EndIf
      If Source > 9
        Source = 0
      EndIf 
      If Source < 0
        Source = 9
      EndIf 
      If KeyboardReleased(#PB_Key_E) ;Erhht Destination-Wert von SpriteBlendingMode der 'Punkte'
        Destination + 1
      EndIf 
      If KeyboardReleased(#PB_Key_D) ;Verringert Destination-Wert von SpriteBlendingMode der 'Punkte'
        Destination - 1
      EndIf
      If Destination > 9
        Destination = 0
      EndIf
      If Destination < 0
        Destination = 9
      EndIf
      
      If KeyboardReleased(#PB_Key_R) ;Erhht Source-Wert von SpriteBlendingMode der 'Ellipse'
        SourceE + 1
      EndIf 
      If KeyboardReleased(#PB_Key_F) ;Verringert Source-Wert von SpriteBlendingMode der 'Ellipse'
        SourceE - 1
      EndIf
      If SourceE > 9
        SourceE = 0
      EndIf 
      If SourceE < 0
        SourceE = 9
      EndIf 
      If KeyboardReleased(#PB_Key_T) ;Erhht Destination-Wert von SpriteBlendingMode der 'Ellipse'
        DestinationE + 1
      EndIf 
      If KeyboardReleased(#PB_Key_G) ;Verringert Destination-Wert von SpriteBlendingMode der 'Ellipse'
        DestinationE - 1
      EndIf
      If DestinationE > 9
        DestinationE = 0
      EndIf
      If DestinationE < 0
        DestinationE = 9
      EndIf
      
      If KeyboardPushed(#PB_Key_Left) ;Wechsle durch verfgbare 'Aufnahmegerte' (z.B: Mikrofon oder Stereomix)
        DevMinus + 1
      EndIf 
      If KeyboardReleased(#PB_Key_Left) And DevMinus > 0
        Dev - 1
        changedDev = 1
        DevMinus = 0
      EndIf
      
      If KeyboardPushed(#PB_Key_Right) ;Wechsle durch verfgbare 'Aufnahmegerte' (z.B: Mikrofon oder Stereomix)
        DevPlus + 1
      EndIf 
      If KeyboardReleased(#PB_Key_Right) And DevPlus > 0
        Dev + 1
        changedDev = 1
        DevPlus = 0
      EndIf 
      
      If Dev < 0
        Dev = NumDevsMax
      EndIf
      If Dev > NumDevsMax
        Dev = 0
      EndIf
      
      If changedDev = 1
        SetWindowCallback(@CAPTURE_CALLBACK())
        CAPTURE_START()
        changedDev = 0
      EndIf 
      
      
      
      If KeyboardReleased(#PB_Key_Escape)
        Quit = 1
      EndIf
      ;   Delay(Delay)
    Until Quit = 1 
    
    ;shutdown:
    For i = 0 To Config\nBuf - 1
      FreeMemory(inHdr(i)\lpData) ;FREE ALLOCATED MEMORY
    Next
    
    If Source <> StartSource
      AndereSource = 1
    EndIf
    If Destination <> StartDestination
      AndereDestination = 1
    EndIf
    If SourceE <> StartSourceE
      AndereSourceE = 1
    EndIf
    If DestinationE <> StartDestinationE
      AndereDestinationE = 1
    EndIf
    If Mode <> StartMode
      AndereMode = 1
    EndIf 
    If Dev <> StartDev
      AndereDev = 1
    EndIf 
    
    
    If AndereSource = 1 Or AndereDestination = 1 Or AndereSourceE = 1 Or AndereDestinationE = 1 Or AndereMode = 1 Or AndereDev = 1
      MessageRequester("Variablen wurden gendert","Startwerte waren: "+ #LF$+"Source: "+Str(StartSource)+ #LF$+"Destination: "+Str(StartDestination)+ #LF$+"SourceE(llipse): "+Str(StartSourceE)+ #LF$+"DestinationE(llipse): "+Str(StartDestinationE)+ #LF$+ "Farbmodus: " +Str(StartMode) +#LF$+ "Input: " +Str(StartDev) +#LF$+#LF$+ #LF$+"Genderte Werte sind:"+ #LF$+"Source: "+Str(Source)+ #LF$+"Destination: "+Str(Destination)+ #LF$+"SourceE(llipse): "+Str(SourceE)+ #LF$+"DestinationE(llipse): "+Str(DestinationE)+#LF$+"Farbmodus: "+Str(Mode)+#LF$+ "Input: " +Str(Dev))
      Result = MessageRequester("Mchten Sie die genderten Werte speichern?","Wenn Sie die Werte speichern, startet das Programm beim nchsten Mal mit diesen Werten.",#PB_MessageRequester_YesNo)
      If Result = #PB_MessageRequester_Yes
        CreateFile(0,"StartSettings.txt")
        WriteLong(0,Dev)
        WriteLong(0,Dev)
        WriteLong(0,Mode)
        WriteLong(0,Source)
        WriteLong(0,Destination)
        WriteLong(0,SourceE)
        WriteLong(0,DestinationE)
        WriteLong(0,Mode)
        CloseFile(0)
      EndIf
    EndIf 
  EndIf
EndIf

End
; IDE Options = PureBasic 5.61 (Windows - x64)
; CursorPosition = 290
; FirstLine = 84
; Folding = 5
; EnableXP
; Executable = Visualizeee.exe
; DisableDebugger